// CIDSQL.cpp : Defines the entry point for the console application.
// Thanks to Dave Merner for his initial classes.
// Author: Ian Duff 
// Date  : December 17 2001
// Email : ianduff@bold.net.au
// Important: PLEASE READ THE NOTES BELOW!
//

/********************************************************************
* Notes: This demonstration WILL work correctly IF:
*
* 1. This example uses the sample Northwind access database that
*    SHOULD be available on most PC's as Nortwind.mdb.
*
* 2. If you have an ODBC connection called Northwind, use that one
*    as this example connects to the source and extracts employees
*    from it using the following query:
*    SELECT *
*    FROM Employees
* 
* 3. This example differs from Dave Merner's class by:
*   - Using the STL string class instead of MFC.
*   - Deprecated ODBC calls have been replaced with current ODBC calls
*   - The MFC CPtrArray class has been replaced by STL vectors.
*   - The CIDSQLColumn class complies with STL requirements (use as
*     objects in container classes) by containing:
*     i) Default constructor
*    ii) Default destructor
*   iii) Copy constructor
*    iv) Assignment operator
*     v) operator<()
*    vi) operator==()
* 4. The output from this example is identical to the output provided
*    by D. Merner's SQLDirect classes.
*********************************************************************/

#include "stdafx.h"
#include "IDSQLDirect.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

/********************************************************************
* Function      : DisplayData
* Description   : Displays all data rows to the screen.
* Parameters    : sqlDirect - A populated CSQLDirect reference.
* Returns       : SQL_SUCCESS (0) = success, otherwise error.
* Notes         : None
*********************************************************************/
static int DisplayData( CIDSQLDirect &sqlDirect )
{
	int nRetCode = 0;
	sqlDirect.PrintColumnNames();
	string strFetch;

    int nColumns = sqlDirect.GetNumColumns();
    cout << "Query returned (" << nColumns << ") columns" << endl;

    while(( sqlDirect.Fetch()) == SQL_SUCCESS ) {
		strFetch.erase();
        for (int i = 1; i <= nColumns; i++ ) {
                strFetch += "\"";
                strFetch += sqlDirect.GetCol( i );
                if ( i == nColumns )
                    strFetch += "\"";
                else strFetch += "\",";
        }
		copy( strFetch.begin(), strFetch.end(), ostream_iterator<char>(cout, "" ) );
		cout << endl;
    }
	return( nRetCode );
}

/********************************************************************
* Function      : SelectEmployees
* Description   : Selects all employees from the northwind database.
* Parameters    : None.
* Returns       : SQL_SUCCESS (0) = success, otherwise error.
* Notes         : None
*********************************************************************/
static int SelectEmployees()
{
	int nRetCode = 0;

    try {
    /*
    * SQLConnect does not use a connection string; using SQLConnect  is 
    * analogous to connecting using a connection string with exactly three
    * keyword/value pairs (for data source name and, optionally, user ID 
    * and password).
    */
		cout << "Connecting to Sample Northwind database ..." << endl;	
		CIDSQLDirect sqlDirect;     // Our direct ODBC class

		if (( nRetCode = sqlDirect.Connect("Northwind") ) != SQL_SUCCESS )
			return( nRetCode );

        if (( nRetCode = sqlDirect.ExecuteSQL("SELECT COUNT(*) FROM Employees") ) != SQL_SUCCESS )
			return( nRetCode );

        if (( nRetCode = sqlDirect.Fetch() ) != SQL_SUCCESS )
			return( nRetCode );	// Fetch the result from this COUNT(*)

        string strFetch = sqlDirect.GetCol( 1 );
        cout << "Employees Table contains (" << strFetch << ") entries" << endl;

        // This query returns all data in the Employees table
        if (( nRetCode = sqlDirect.ExecuteSQL("SELECT * FROM Employees")) != SQL_SUCCESS )
			return( nRetCode );

        int nColumns = sqlDirect.GetNumColumns();
        cout << "Query returned (" << nColumns << ") columns" << endl;
		
		sqlDirect.PrintColumnNames();
		DisplayData( sqlDirect );
    }
    catch( ... ) {
        cout << "SQL Exception Caught!" << endl;
    }
	return( nRetCode );
}

/********************************************************************
* Function      : ExtendedOrderDetails
* Description   : Reports extended order details from the northwind
*                 database.
* Parameters    : None.
* Returns       : SQL_SUCCESS (0) = success, otherwise error.
* Notes         : This is almost exactly the same as the Access db.
*********************************************************************/
static int ExtendedOrderDetails()
{
	int		nRetCode = 0;
	string	strFetch;
	string	strSQL   = "SELECT DISTINCTROW [Order Details].OrderID, "
						"[Order Details].ProductID, "
						"Products.ProductName, "
						"[Order Details].UnitPrice, "
						"[Order Details].Quantity, "
						"[Order Details].Discount, "
						"CCur([Order Details].[UnitPrice]*[Quantity]*(1-[Discount])/100)*100 AS ExtendedPrice "
						"FROM Products "
						"INNER JOIN [Order Details] "
						"ON Products.ProductID = [Order Details].ProductID "
						"ORDER BY [Order Details].OrderID;" ;

	try {
		cout << "Connecting to Sample Northwind database ..." << endl;
        CIDSQLDirect sqlDirect;

		if (( nRetCode = sqlDirect.Connect("Northwind") ) != SQL_SUCCESS )
			return( nRetCode );

		if (( nRetCode = sqlDirect.ExecuteSQL( strSQL.c_str() ) ) != SQL_SUCCESS )
			return( nRetCode );

		sqlDirect.PrintColumnNames();
		DisplayData( sqlDirect );

	}
	catch( ... ) {
		cout << "SQL exception caught!" << endl;
	}
	return( nRetCode );
}

/********************************************************************
* Function      : ComplicatedInvoiceQuery
* Description   : A complicated invoice query example
* Parameters    : None.
* Returns       : SQL_SUCCESS (0) = success, otherwise error.
* Notes         : 
* 1. From the Northwind mdb query.
* 2. Returns 2155 rows.
*********************************************************************/
static int ComplicatedInvoiceQuery()
{
	int nRetCode = 0;
	CIDSQLDirect sqlDirect;
	string strFetch;
	string strSQL =	"SELECT DISTINCTROW "
					"Orders.ShipName, "
					"Orders.ShipAddress, "
					"Orders.ShipCity, "
					"Orders.ShipRegion, "
					"Orders.ShipPostalCode, "
					"Orders.ShipCountry, "
					"Orders.CustomerID, "
					"Customers.CompanyName, "
					"Customers.Address, "
					"Customers.City, "
					"Customers.Region, "
					"Customers.PostalCode, "
					"Customers.Country, "
					"[LastName] AS Salesperson, "
					"Orders.OrderID, "
					"Orders.OrderDate, "
					"Orders.RequiredDate, "
					"Orders.ShippedDate, "
					"Shippers.CompanyName, "
					"[Order Details].ProductID, "
					"Products.ProductName, "
					"[Order Details].UnitPrice, "
					"[Order Details].Quantity, "
					"[Order Details].Discount, "
					"CCur([Order Details].[UnitPrice]*[Quantity]*(1-[Discount])/100)*100 AS ExtendedPrice, "
					"Orders.Freight "
					"FROM Shippers "
					"INNER JOIN (Products "
					"INNER JOIN ((Employees "
					"INNER JOIN (Customers "
					"INNER JOIN Orders "
					"ON Customers.CustomerID = Orders.CustomerID) "
					"ON Employees.EmployeeID = Orders.EmployeeID) "
					"INNER JOIN [Order Details] "
					"ON Orders.OrderID = [Order Details].OrderID) "
					"ON Products.ProductID = [Order Details].ProductID) "
					"ON Shippers.ShipperID = Orders.ShipVia;";
	try {
		cout << "Connecting to Sample Northwind database ..." << endl;

		if (( nRetCode = sqlDirect.Connect("Northwind") ) != SQL_SUCCESS )
			return( nRetCode );

		if (( nRetCode = sqlDirect.ExecuteSQL( strSQL.c_str() ) ) != SQL_SUCCESS )
			return( nRetCode );

		sqlDirect.PrintColumnNames();
		DisplayData( sqlDirect );

	}
	catch( ... ) {
		cout << "SQL exception caught!" << endl;
	}
	return( nRetCode );
}

/********************************************************************
* Function      : DSNConnections
* Description   : Executes ODBC SQL commands through a DSN string.
* Parameters    : None.
* Returns       : SQL_SUCCESS (0) = success, otherwise error.
* Notes         : You will probably need to change the dsn below to
*                 suit your particular application.
*********************************************************************/
static int DSNConnections()
{
	int nRetCode = 0;
	string strDSN = "DSN=NorthWind;DBQ=C:\\DataBase\\Northwind.mdb;DriverId=281;FIL=MS Access;MaxBufferSize=2048;PageTimeout=5;UID=;";
    try {
        cout << "Connecting to Sample Northwind database ..." << endl;
        CIDSQLDirect sqlDirect;

		if (( nRetCode = sqlDirect.ConnectDSN( strDSN ) ) != SQL_SUCCESS )
			return( nRetCode );

		if (( nRetCode = sqlDirect.ExecuteSQL( "SELECT COUNT(*) FROM Employees" ) ) != SQL_SUCCESS )
			return( nRetCode );

        if (( nRetCode = sqlDirect.Fetch() ) != SQL_SUCCESS )
			return( nRetCode );

        string strFetch = sqlDirect.GetCol( 1 );
        cout << "Employees Table contains (" << strFetch << ") entries" << endl;

        // This query returns all data in the Employees table
		if (( nRetCode = sqlDirect.ExecuteSQL( "SELECT * FROM Employees" ) ) != SQL_SUCCESS )
			return( nRetCode );

        int nColumns = sqlDirect.GetNumColumns();
        cout << "Query returned (" << nColumns << ") columns" << endl;
		
		sqlDirect.PrintColumnNames();
		DisplayData( sqlDirect );
    }
    catch( ... ) {
        cout << "SQL Exception Caught!" << endl;
    }
	return( nRetCode );
}

/********************************************************************
* Function      : ConnectDifferently
* Description   : Example of setting user name, password and connect
* Parameters    : None.
* Returns       : SQL_SUCCESS (0) = success, otherwise error.
* Notes         : None.
*********************************************************************/
static int ConnectDifferently()
{
    int          nRetCode = 0;
    CIDSQLDirect sqlDirect;
    try {
        // Set password,user name and immediately connect
        sqlDirect.SetUserName("").SetPassword("").Connect("NorthWind");

        if (( nRetCode = sqlDirect.ExecuteSQL("SELECT COUNT(*) FROM Employees") ) != SQL_SUCCESS )
			return( nRetCode );

        if (( nRetCode = sqlDirect.Fetch() ) != SQL_SUCCESS )
			return( nRetCode );	// Fetch the result from this COUNT(*)

        string strFetch = sqlDirect.GetCol( 1 );
        cout << "Employees Table contains (" << strFetch << ") entries" << endl;

        // This query returns all data in the Employees table
        if (( nRetCode = sqlDirect.ExecuteSQL("SELECT * FROM Employees")) != SQL_SUCCESS )
			return( nRetCode );

        int nColumns = sqlDirect.GetNumColumns();
        cout << "Query returned (" << nColumns << ") columns" << endl;
		
		sqlDirect.PrintColumnNames();
		DisplayData( sqlDirect );

    }
    catch( ... ) {
        cout << "SQL Exception Caught!" << endl;
    }
    return( nRetCode );
}

/********************************************************************
* Funtion       : main
* Description   : Entry point into the program.
* Parameters    :
* 1. int - The number of arguments
* 2. argv - The command line args themselves.
* 3. envp - An array of environment variables.
* Returns       : 0 = success, otherwise fail.
*
* Notes :
* The use of the CIDSQLDirect class has been modified as per point 3
* in the supplied notes at the top of this file. PLEASE READ THEM!
* because if you don't have a DSN set up for your sample Northwind
* database, this program will fail.
*********************************************************************/
int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0)) {
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		return( nRetCode = 1 );
	}

	// Use simple select query for Employees
	if (( nRetCode = SelectEmployees() ) != SQL_SUCCESS )
		return( nRetCode );
	
	// Now a slightly more complicated example query
	if (( nRetCode = ExtendedOrderDetails() ) != SQL_SUCCESS )
		return( nRetCode );

	// An even more complicated example
	if (( nRetCode = ComplicatedInvoiceQuery() ) != SQL_SUCCESS )
		return( nRetCode );

	// A simple employee select through a DSN connection string
	if (( nRetCode = DSNConnections() ) != SQL_SUCCESS )
		return( nRetCode );

    // Demonstrate another way of connecting
    if (( nRetCode = ConnectDifferently() ) != SQL_SUCCESS )
        return( nRetCode );

	return( nRetCode );
}


