// IDSQLColumn.h: interface for the CIDSQLColumn class.
//
// This column class has been represented in orthodox canonical form of
// class and contains operator functions for use with STL containers, etc.
//
// Author: Ian Duff - 30 December 2001
// Email : ianduff@bold.net.au
// Thanks to: 
// Dave Merner - direct SQL calls with ODBC
//
//////////////////////////////////////////////////////////////////////

#if !defined(   __IDSQLCOLUMN_H__ )
#define         __IDSQLCOLUMN_H__ 

// ---------- Module includes

#include <string>
#include <algorithm>

// ---------- Module namespaces 

using namespace std;

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/******************************************************************************
* This class has been modified to include the necessary operators
* required as canonical form of class for use with STL containers,etc
*
* As such, you may now use the STL algorithms, eg sort():
*
******************************************************************************/

class CIDSQLColumn  
{

public:
    /*
    * Default constructor
    */
	CIDSQLColumn();      

    /*
    * Copy constructor
    */
    CIDSQLColumn( const CIDSQLColumn& );

    /*
    * Assignment operator
    */
    CIDSQLColumn& operator=( const CIDSQLColumn& );


    /*
    * Default destructor
    */
	virtual ~CIDSQLColumn();    // Default destructor
    
    /*
    * operator< required for sorting, etc
    */
    bool operator<( const CIDSQLColumn& ); 


    /*
    * also used by container classes
    */
    bool operator==( const CIDSQLColumn& ) ;

public:

    int         m_nType;    // Column type
	int         m_nCol;     // Column number
    string      m_strValue; // Column value
	
};

#endif // !defined( __IDSQLCOLUMN_H__ )
